// --- [ ds1631.c ] -------------------------------------------------------------
//			Routinen zur Temperaturmessung mit dem DS1631, (tab = 3)

#include <string.h>
#include <stdlib.h>
#include <inttypes.h>
#include "TWI_master_a.h"
#include "ds1631.h"
#include "global.h"

// Adressen DS1631:								0x90, (144) bis 0x9E (158)

uint8_t DS1631_buffer[3];						// Zur Datenbergabe an das TWI-Modul
//int8_t temphi, templo;						// lokale Variablen fr die Temperaturbytes

// ----------------------------------------------------------------------------
// Initialisiert den Chip mit der Konstanten DS1631_CONFIG
void ds1631_Init(uint8_t chipadr)
{
	DS1631_buffer[0] = chipadr;									// Chipadresse
	DS1631_buffer[1] = DS1631_POR;								// Reset
	TWI_Start_Transceiver_With_Data( DS1631_buffer, 2 );	// schreiben
   	
	DS1631_buffer[1] = DS1631_ACCESS_CONFIG;					// Config
	DS1631_buffer[2] = DS1631_CONFIG;							// Config-Bits
	TWI_Start_Transceiver_With_Data( DS1631_buffer, 3 );	// schreiben
}

// ----------------------------------------------------------------------------
// Liest den Status und gibt den Wert zurck
uint8_t ds1631_Get_Status(uint8_t chipadr)
{
	DS1631_buffer[0] = chipadr;									// Chipadresse
	DS1631_buffer[1] = DS1631_ACCESS_CONFIG;					// 	
	TWI_Start_Transceiver_With_Data( DS1631_buffer, 2 ); 	// schreiben
	
	DS1631_buffer[0]++;
	TWI_Start_Transceiver_With_Data( DS1631_buffer, 2 ); 	// lesen
	TWI_Get_Data_From_Transceiver( DS1631_buffer, 2 );

	return( DS1631_buffer[1]);
}

// ----------------------------------------------------------------------------
// Stt einen Messvorgang an, fr die notwendige Verzgerung bis zum 
// Einlesen des Messergebnis muss die aufrufende Routine sorgen
void ds1631_Read_Start(uint8_t chipadr)
{
	DS1631_buffer[0] = chipadr;									// Chipadresse
	DS1631_buffer[1] = DS1631_START_CONVERT;					// Messung auslsen	
	TWI_Start_Transceiver_With_Data( DS1631_buffer, 2 ); 	// schreiben
}

// ----------------------------------------------------------------------------
// Liest das Messergebnis in die Variablen temphi und templo
// nach dem Read_Start muss ca. 1Sekunde gewartet werden, 
// bis das Ergebnis zum Auslesen bereit steht.
uint16_t ds1631_Read(uint8_t chipadr)
{
	while (!(ds1631_Get_Status(chipadr) & 128));	// warten, solange wie Bit.7 nicht gesetzt

	DS1631_buffer[0] = chipadr;	
	DS1631_buffer[1] = DS1631_READ_T;		// Command READ_T ausgeben
	TWI_Start_Transceiver_With_Data( DS1631_buffer, 2 ); // schreiben
		
	DS1631_buffer[0]++;							// Adresse zum Lesen incrementieren
						
	TWI_Start_Transceiver_With_Data( DS1631_buffer, 3 ); // lesen
	TWI_Get_Data_From_Transceiver( DS1631_buffer, 3 );
		
	// der ganzzahlige Anteil der Temperatur steht in DS1631_buffer[1]
//	temphi = DS1631_buffer[1];	
//	templo =	DS1631_buffer[2];					// Werte auch in lokaler Variable speichern

//	temphi = 0x45;
//	templo = 0xAA;
return((DS1631_buffer[1] << 8) + DS1631_buffer[2]);

}

/* // ---------------------------------------------------------------------------
// baut einen Textstring mit der formatierten Temperaturanzeige auf
// der String hat ist immer 5 Zeichen lang, ggf. wird ein fhrendes Leerzeichen vorangestellt
void ds1631_Get_String(char *s)
{
uint8_t t;
uint16_t temp16;									
											
for(t = 0; t < 6; t++) s[t] = 0;				// den String mit 0 fllen

if (temphi >= 10)									// ein "+" voranschreiben, wenn >= 10
	{
	s[0] = '+';
	}

else if (temphi >= 0)							// ein " +" voranschreiben, wenn >= 0
	{
	s[0] = ' ';
	s[1] = '+';
	}

else if ((temphi > -2) && templo)			// fr -0.x Grad, hier negatives Vorzeichen setzen
	{													// aber nicht bei genau -1
	s[0] = ' ';
	s[1] = '-';
	}

else if (temphi > -10) 							// fr 0 bis -9 ein Leerzeichen ausgeben
	{
	s[0] = ' ';										// das Vorzeichen kommt von itoa()
	}

templo &= 0xF0;									// die Werte stehen im upper nibble
if (temphi < 0) 									// bei negativen Werten (Vorkomma) wirds kompliziert ...
	{
	templo = 0 - templo;							//	bei negativen Temperaturen Zweierkomplement bilden
	if (templo & 0xF0)							// wenn Nachkommastellen vorhanden
		{												// dann eine 1 zur Vorkommastelle addieren 
		temphi++;									// z.B. -3 + 1 = -2 (und daran kommen die Nachkommastellen)
		}
	}

temp16 = (uint16_t) templo * (640 / 16);	// Nachkommastelle berechnen
temp16 /= 1024;									// insgesamt mit 0.625 multiplizieren

s = strcat(s, itoa(temphi, msg, 10));		// das negative Vorzeichen kommt von hier
s[4] = '.';	
s[5] = ((int8_t) temp16 + 0x30);				// die Nachkommastelle besteht nur aus 1 Ziffer

} */


// --- [ eof ] ----------------------------------------------------------------
