// --- [ logger.c ] -----------------------------------------------------------
//       die "passive" Variante des Loggers
//			nimmt Daten via TWI oder RS232 entgegen und speichert sie auf dem Dataflash
//       tab = 3

#define EXTERN

#include <avr/io.h>
#include <string.h>	
#include <stdlib.h>
#include <util/delay.h>
#include <avr/interrupt.h>

//#include "vnc_spi.h"
#include "rs232.h"
#include "spi_hard.h"
#include "global.h"
#include "TWI_df_slave.h"
#include "at45db081.h"

// --- public - Variablen -----------------------------------------------------

#define LOG_TWI									// via TWI   loggen
#define LOG_RS232									// via RS232 loggen

void record(void);
void play(void);
void increment_seconds(void);


// ----------------------------------------------------------------------------
int main(void)
{

CLKPR = (1<<CLKPCE);								// Clockprescale = 1
CLKPR = 0; 
														// Strom sparen, Timer2/1 und ADC abschalten
PRR = (1<<PRTIM2 | 1<<PRTIM1 | 0<<PRTIM0 | 0<<PRSPI | 0<<PRUSART0 | 1<<PRADC);

// Der Timer0 wird im Clear Timer on Compare Match Modus betrieben
// Sobald der TCTN0 den Wert in OCR0A berschreiten, wird er TCNT0
// auf 0 zurckgesetzt und eine ISR angesprungen.
// Hier wird allgemein die Zeit aktualisiert sowie die Variable VNC_timeout decrementiert
TCCR0A  = (1<<WGM01);							// CTC-Modus
TCCR0B  = (1<<CS02 | 1<< CS00);				// prescale 1024
TIMSK0  = (1<<OCIE0A);							// enable COMPA
OCR0A   = 120;

// Die abgelaufene Zeit wird sekundenweise in einem 16Bit-Zhler aufgezeichnet
// Jeder Page-Eintrag erhlt diese zwei Byte als Zeiteintrag zugeordnet
time = 0;

LED_DDR  = (1<<LED1 | 1<<LED2 | 1<<LED3);	// Port fr LED als Ausgang
LED1_ON;												// alle LEDs ein
LED2_ON;
LED3_ON;
														// fr einen Taster zum Starten/Stoppen des Loggens/ndern der Zeit
KEY_PORT |= ( 1<<KEY_START | 1<<KEY_PLAY);// Pullups setzen

RS232_init();										// RS232 initialisieren
SPI_init();
TWI_init();
TWI_ENABLE;											// zum Debuggen

asm volatile ("sei");							// fr Timer und auch RX/TWI/SPI erforderlich

SPI_init();											// Initialisierung, danach braucht der DF 20ms

LED_timeout = 15;									// 1/2 Sekunde warten
while (LED_timeout);

if (!(DF_device_ready()))						// wenn dann das DF nicht bereit ist
	{
	LED1_OFF;
	LED3_OFF;
	while(1)											// dann liegt wohl ein Problem vor
		{
		LED_timeout = 7;
		while (LED_timeout);						// nur LED2 toggelt
		LED2_TOG;
		}
	}

// --- Hauptschleife ----------------------------------------------------------
//     Vorbereitung: 
//		 das Dateisystem durchsuchen, die letzte File-ID bestimmen (DF_file_id)
//     die nchste frei Page finden,
//     die Anzahl der freien und belegten Pages ermitteln (for fun)
//
//     diese Phase durch blinkende LED markieren
//
//     danach Warten auf Startzeichen - Taste START oder PLAY

DF_check();											// hier wird die nchste freie Seite ermittelt
DF_erase_page(df_next_page);					// die nchste zu benutzende Seite vorbereiten

LED1_OFF;											
LED2_ON;												// Nur LED2 ist eingeschaltet
LED3_OFF;							

while(1)
	{	
	if (KEY_START_PRESSED)						// wenn die START-Taste gedrckt wurde
		{
		LED_timeout = 10;							// 1/3 Sekunde Wartezeit
		while(LED_timeout);						// warten
		while((KEY_START_PRESSED) && (KEY_PLAY_RELEASED));	// warten bis die Start-Taste losgelassen wird
														// oder zustzlich die Play-Taste gedrckt wird
														
		if ((KEY_PLAY_RELEASED))				// wenn ist die Starttaste losgelassen worden
			{
			record();								// wenn immer noch alleine gedrckt, dann zur Aufnahme springen

			LED1_OFF;	
			LED2_ON;									// Nur LED2 ist eingeschaltet
			LED3_OFF;
			}
		}

	else if (KEY_PLAY_PRESSED)					// wenn die PLAY-Taste gedrckt wurde
		{
		LED_timeout = 10;							// 1/3 Sekunde Wartezeit
		while(LED_timeout);						// warten
		while((KEY_PLAY_PRESSED) && (KEY_START_RELEASED));	// warten bis die Play-Taste losgelassen wird
														// oder zustzlich die Starttaste gedrckt wird
 
		if (KEY_START_RELEASED)					// dann wurde die Play-Taste losgelassen 
			{
			play();									// wenn immer noch alleine gedrckt, dann zur Wiedergabe springen
			
			LED1_OFF;	
			LED2_ON;									// Nur LED2 ist eingeschaltet	
			LED3_OFF;
			}
		}
		

	if ((KEY_PLAY_PRESSED) && (KEY_START_PRESSED)) // wenn START- und PLAY-Taste gleichzeitig gedrckt wurden
		{

		LED_timeout = 50;							// noch einmal 5/3 Sekunde Wartezeit insgesamt also 2 Sekunden
		while(LED_timeout);						// warten
		
		if ((KEY_PLAY_PRESSED) && (KEY_START_PRESSED)) // wenn die Tasten immer noch gedrckt sind,
			{
			DF_erase_chip();						// dann den ganzen Chip lschen
			DF_check();								// und neu die Startdaten ermitteln
			
			LED1_OFF;
			LED2_ON;									// Nur LED2 ist eingeschaltet	
			LED3_OFF;
			}
		}
	}
}


// ----------------------------------------------------------------------------
// hier werden Daten aufgezeichnet
// die LED1 toggelt langsam - aber stetig
// die LED2 toggelt nach jedem geschriebenen Datensatz
// Beendet wird durch Drcken der START-Taste
void record(void)
{
uint8_t i;

rx_buffer_ptr = 0;								// Empfangspuffer 1
rx_idx = 0;											// Index auf den Anfang
file_page_cnt = 1;								// Zhler fr die Anzahl der Pages
time = 0;											// die Zeitzhlung zurcksetzen
file_id++;											// das ist die nchste File-ID

#ifdef LOG_RS232
	RX_ENABLE;										// Datenannahme via RX  enablen 
#endif
#ifdef LOG_TWI 
	TWI_ENABLE; 									// Datenannahme via TWI enablen
#endif

LED1_ON;												// LED1 ist an, LED2 toggelt permanent,
														// LED3 toggled nach dem Schreiben einer Page
// die ID incrementieren
// den Pagecounter zurcksetzen

session_pages = 0;								// Counter fr die geschriebenen Pages der Sitzung
														// wenn die Gesamtblockzahl erreicht ist,
														// dann kann weitere Speicherung beendet werden, um nicht
														// den Anfang der Aufzeichnung zu berschreiben
// --- innere Hauptschleife ----------------------------------------------------------

while(1)												// Schleife fr den Empfang von Daten
	{
	if (!(LED_timeout)) 							// nach 1 Sekunde ist VNC_timeout vom Timer auf 0 decrementiert
		{
		LED_timeout = 15;							// LED1 blinkt mit 1 Hz whrend des Loggens
		LED2_TOG;									
		}
			
	if (rx_flag) 									// wenn der Puffer voll ist
		{
		rx_flag = FALSE;
		bytes_received = 255;					// 256 (!) Bytes sind im Puffer
		i = (rx_buffer_ptr ^1);					// die Daten stehen im anderen Buffer
		DF_write_page(i);							// als Parameter wird bergeben, welcher Buffer zu schreiben ist
		LED3_TOG;
		}

														// Wenn die Start-Taste erneut bettigt wird, 
	if (KEY_START_PRESSED)						// dann wird die geffnete Datei wieder geschlossen 
		{
		
#ifdef LOG_RS232
		RX_DISABLE;									// Datenannahme via RX  abschalten
#endif
#ifdef LOG_TWI 
		TWI_DISABLE; 								// Datenannahme via TWI abschalten
#endif
														// steht rx_idx auf 0, dann hat es gerade einen vollen Puffer
														// gegeben, die Daten sind gesichert, aber kein weiteres
														// Byte ist empfangen worden, daher
		if (rx_idx)									// nur wenn Bytes im Puffer sind, dann wird auch gespeichert
			{
			bytes_received = (rx_idx - 1);	// der Index ist schon fr das nchste Byte incrementiert worden !!
														// die Anzahl der empfangenen Bytes ist um 1 niedriger
			i = (rx_buffer_ptr);					// die Daten stehen im aktuellen Buffer
			DF_write_page(i);						// als Parameter wird bergeben, welcher Buffer zu schreiben ist
			}
		LED_timeout = 3;							// kurze Pause
		while (LED_timeout);

		while(KEY_START_PRESSED);				// warten bis die Taste losgelassen wird
		return;
		}
	}
}

// ----------------------------------------------------------------------------
// spielt die letzte Aufnahme ab und sendet sie via RS232 ab.
// Nach Abschluss der bertragung wartet das Programm auf Tastenbettigung:
// Ein erneutes Drcken der PLAY-Taste gibt die nchst ltere Datei aus
// - bis keine Datei mehr gefunden wird
// Ein Bettigen der START-Taste springt ins Hauptmenue zurck.
void play(void)
{
uint16_t page, startpage, ii;
uint16_t pages_in_file;					
uint8_t file_id_low;								// low-Byte der File_id
uint8_t bytes_in_page, i;

RX_ENABLE;											// die Schnittstelle enablen fr die Ausgabe

// df_next_page zeigt auf die nchste frei Page
// den Wert kopieren, die Variable df_next_page bleibt erhalten
startpage = df_next_page;
// die letzte Aufnahme wird durch ihre ID charakterisiert.
// es muss also solange zurckgespult werden, wie die ID identisch ist.
// zustzlich ist die Anzahl der Pages ja kodiert und kann zur Berechnung des
// Anfanges herangezogen werden, wenn man die Kette von hinten her aufdrselt

LED1_OFF;											// LED1 aus
														// LED2 blinkt
LED3_ON;												// LED3 auf Dauerlicht
while(1)
	{
// das ist die letzte Page der vorhergehenden Datei
	startpage--;						 
// die Page-Nr. luft von 0, 1 bis 4094, 4095
	startpage &= PAGE_MASK;						// verhindert Over/Underflow
	
// Start dieser Datei ermitteln
// aktuelle Pagezahl - Anzahl der Pages der Datei
	DF_read_byte(startpage, 256, 8);
// Jetzt steht der Header des letzen Datensatzes in df_buffer[]
	pages_in_file  = df_buffer[P_PAGE_HIGH] << 8;
	pages_in_file += df_buffer[P_PAGE_LOW];
	
	file_id_low = df_buffer[P_ID_LOW];		// nur das low-Byte der id merken

// hier sollte der Anfang der Datei sein
	startpage -= (pages_in_file - 1) ;		// Filesize beginnt mit 1						
	startpage &= PAGE_MASK;						// verhindert Underflow

// Dorthin springen und den Header einlesen,
// wenn dort dieselbe fileid vorliegt und
// wenn filesize dort 1 ist, dann stimmt das Ziel
	DF_read_byte(startpage, 256, 8);

// die FileId (low_Byte) vergleichen, wenn sie nicht bereinstimmt,
// dann liegt ein Fehler vor.
	if (!(file_id_low == df_buffer[P_ID_LOW]))
		{
		for (i = 0; i < 50; i++)
			{
			LED_timeout = 3;
			while (LED_timeout);
			LED2_TOG;
			}
		return;
		}

// jetzt prfen, ob als filesize tatschlich 1 eingetragen ist.
// wenn nein, dann liegt ein Fehler vor. 	
	if (!((df_buffer[P_PAGE_HIGH] == 0) && (df_buffer[P_PAGE_LOW] == 1)))
		{
		for (i = 0; i < 50; i++)
			{
			LED_timeout = 3;
			while (LED_timeout);
			LED2_TOG;
			}
		return;	
		}
// wenn der Eintrag fr filesize wie erwartet == 1 ist, dann die Daten ausgeben
	else
		{
// die startpage merken, fr den Fall, das weitere Dateien gelesen werden sollen
		page = startpage;
// Pageweise die Daten in den internen Buffer laden
// und via RS232 rausschreiben
// bis zum letzten Block, der wird vermutlich krzer als 256 Byte sein

		for (ii = 0; ii < pages_in_file; ii++)
			{
// die letzte Page ist wahrscheinlich nur teilweise mit Daten belegt	
			DF_read_byte(page, 256, 2);
			bytes_in_page = df_buffer[1];    // Anzahl der Bytes holen
			
			DF_read_byte(page, 256, 8);		// Druckt zum Debuggen den Header aus
			RS232_print_header(page);

			DF_read_byte(page, 0, 255);		// die Daten nach df_buffer[] einlesen
			RS232_print_page(df_buffer, bytes_in_page);		// und wohlformatiert ausgeben
		
			page++;
			page &= PAGE_MASK;					// gegen Overflow am Ende des Speichers
			
			LED2_TOG;								// LED2 toggled nach jeweils 256 Byte
			}
		}
	LED2_OFF;										// Dauerlicht an LED3 zeigt Ende der Ausgabe an

// warten, bis eine der beiden Tasten bettigt wird
	while (KEY_START_RELEASED && KEY_PLAY_RELEASED);

	LED_timeout = 3;
	while (LED_timeout);

// wenn die Starttaste gedrckt wird, dann zurck zur Hauptebene	
	if (KEY_START_PRESSED) 
		{
		RX_DISABLE;
		while(KEY_START_PRESSED);			// warten bis die Taste losgelassen wird
		return;
		}

	while(KEY_PLAY_PRESSED);				// warten bis die Play-Taste losgelassen wird
// wenn die Play-Taste gedrckt war, dann die nchste Datei ausgeben
// startpage zeigt immer noch auf den Anfang der aktuellen Datei
//	put_c('+');									// zum Debuggen
//	put_c(13);
	}
}

// ----------------------------------------------------------------------------
// TIMER0_OVF:  3.686400 MHZ / (1024 * 120) -> 33ms  Zhlt VNC_timeout herunter auf 0
// als Grundlage fr die Zeitzhlung muss ein Takt gewhlt werden, aus dem sich
// der Sekundentakt ableiten lsst: Die ISR wird 30x /Sekunde aufgerufen
ISR(TIMER0_COMPA_vect)
{
static uint8_t tick = 30; 

if(LED_timeout) LED_timeout--;				// timeout runterzhlen

tick--;
if (!(tick))										// wenn 1 Sekunde um ist (tick = 30)
	{												
	tick = 30;
	time++;											// die Zeit im Sekundentakt incrementieren
	}
}

// --- [ eof ] ----------------------------------------------------------------

