// --- [ logger.c ] -----------------------------------------------------------
// DatenLogger mit Atmel DataFlash AT45DB081D ( 1MByte )
// hier als aktiver TWI-Master, der beliebige Slave abfragt und die Daten im externen
// Flash ablegt


#define EXTERN

#include <avr/io.h>
#include <string.h>	
#include <stdlib.h>
#include <avr/interrupt.h>

#include "rs232.h"
#include "spi_hard.h"
#include "global.h"
#include "TWI_master_a.h"
#include "at45db081.h"
#include "ds1631.h"

// --- public - Variablen -----------------------------------------------------

void record(void);
void play(void);
void save_data(uint8_t data);
void increment_seconds(void);

volatile uint8_t LOG_flag;						// signalisiert, dass die Zeit zum Messen gekommen ist

// ----------------------------------------------------------------------------
int main(void)
{

CLKPR = (1<<CLKPCE);								// Clockprescale = 1
CLKPR = 0; 
														// Strom sparen, Timer2/1 und ADC abschalten
PRR = (1<<PRTIM2 | 1<<PRTIM1 | 0<<PRTIM0 | 0<<PRSPI | 0<<PRUSART0 | 1<<PRADC);

// Der Timer0 wird im Clear Timer on Compare Match Modus betrieben
// Sobald der TCTN0 den Wert in OCR0A berschreiten, wird er TCNT0
// auf 0 zurckgesetzt und eine ISR angesprungen.
// Hier wird allgemein die Zeit aktualisiert sowie die Variable VNC_timeout decrementiert
TCCR0A  = (1<<WGM01);							// CTC - Modus 2
TCCR0B  = (1<<CS02 | 1<< CS00);				// prescale 1024
TIMSK0  = (1<<OCIE0A);							// enable COMPA
OCR0A   = 120;

// Die abgelaufene Zeit wird sekundenweise in einem 16Bit-Zhler aufgezeichnet
// Jeder Page-Eintrag erhlt diese zwei Byte als Zeiteintrag zugeordnet
// time = 0; 										// wird zu Beginn eines jeden Logvorgangs auf 0 gesetzt !!

LED_DDR  = (1<<LED1 | 1<<LED2 | 1<<LED3);	// Port fr LED als Ausgang
LED1_ON;												// alle LEDs ein
LED2_ON;
LED3_ON;
														// fr einen Taster zum Starten/Stoppen des Loggens/ndern der Zeit
KEY_PORT |= ( 1<<KEY_START | 1<<KEY_PLAY);// Pullups setzen

RS232_init();										// RS232 initialisieren
SPI_init();
TWI_Master_Init();

asm volatile ("sei");							// fr Timer und auch RX/TWI/SPI erforderlich

SPI_init();											// Initialisierung, danach braucht der DF 20ms

LED_timeout = 15;									// 1/2 Sekunde warten
while (LED_timeout);

if (!(DF_device_ready()))						// wenn dann das DF nicht bereit ist
	{
	LED1_OFF;
	LED3_OFF;
	while(1)											// dann liegt ein Problem vor
		{
		LED_timeout = 7;
		while (LED_timeout);						// nur LED2 toggelt
		LED2_TOG;
		}
	}

// --- Hauptmenue -------------------------------------------------------------
//     Vorbereitung: 
//		 das Dateisystem durchsuchen, die letzte File-ID bestimmen (DF_file_id)
//     die nchste frei Page finden,
//     diese Phase durch blinkende LED markieren
//
//     danach Warten auf Startzeichen - Taste START oder PLAY

DF_check();											// hier wird die nchste freie Seite ermittelt
DF_erase_page(df_next_page);					// die nchste zu benutzende Seite vorbereiten

LED1_OFF;											
LED2_ON;												// Nur LED2 ist eingeschaltet
LED3_OFF;							

while(1)
	{	

	if (KEY_START_PRESSED)						// wenn die START-Taste gedrckt wird
		{
		LED_timeout = 10;							// 1/3 Sekunde Wartezeit
		while(LED_timeout);						// warten
		while((KEY_START_PRESSED) && (KEY_PLAY_RELEASED));	// warten bis die Start-Taste losgelassen wird
														// oder zustzlich die Play-Taste gedrckt wird
														
		if ((KEY_PLAY_RELEASED))				// wenn die Starttaste gedrckt wurde
			{
			record();								// wenn immer noch alleine gedrckt, dann zur Aufnahme springen

			LED1_OFF;	
			LED2_ON;									// Nur LED2 ist eingeschaltet
			LED3_OFF;
			}
		}

	else if (KEY_PLAY_PRESSED)					// wenn die PLAY-Taste gedrckt wurde
		{
		LED_timeout = 10;							// 1/3 Sekunde Wartezeit
		while(LED_timeout);						// warten
		while((KEY_PLAY_PRESSED) && (KEY_START_RELEASED));	// warten bis die Play-Taste losgelassen wird
														// oder zustzlich die Starttaste gedrckt wird
 
		if (KEY_START_RELEASED)					// dann wurde die Play-Taste losgelassen 
			{
			play();									// wenn immer noch alleine gedrckt, dann zur Wiedergabe springen
			
			LED1_OFF;	
			LED2_ON;									// Nur LED2 ist eingeschaltet	
			LED3_OFF;
			}
		}
		
	if ((KEY_PLAY_PRESSED) && (KEY_START_PRESSED)) // wenn START- und PLAY-Taste gleichzeitig gedrckt wurden
		{

		LED_timeout = 50;							// noch einmal 5/3 Sekunde Wartezeit insgesamt also 2 Sekunden
		while(LED_timeout);						// warten
		
		if ((KEY_PLAY_PRESSED) && (KEY_START_PRESSED)) // wenn dann die Tasten immer noch gedrckt sind,
			{
			DF_erase_chip();						// dann den ganzen Chip lschen
			DF_check();								// und neu die Startdaten ermitteln
			
			LED1_OFF;
			LED2_ON;									// Nur LED2 ist eingeschaltet	
			LED3_OFF;
			}
		}
	}
}


// ----------------------------------------------------------------------------
// hier werden Daten aufgezeichnet
// die LED1 toggelt langsam - aber stetig
// die LED2 toggelt nach jedem geschriebenen Datensatz
// Beendet wird durch erneutes Drcken der START-Taste
void record(void)
{
uint8_t twi_slave_present[4];					// hier werden fehlende Slaves eingetragen

uint8_t i,j, chipadr;
uint16_t temperatur;


rx_buffer_ptr = 0;								// Empfangspuffer 1
rx_idx = 0;											// Index auf den Anfang
file_page_cnt = 1;								// Zhler fr die Anzahl der Pages
time = 0;											// die Zeitzhlung zurcksetzen
file_id++;											// das ist die nchste File-ID

LED1_ON;												// LED1 ist an, LED2 toggelt permanent,
														// LED3 toggled jeweils nach dem Schreiben einer Page

session_pages = 0;								// Counter fr die geschriebenen Pages der Sitzung
														// wenn die Gesamtblockzahl erreicht ist,
														// dann kann weitere Speicherung beendet werden, um nicht
														// den Anfang der Aufzeichnung zu berschreiben
														
j = 0;												// Vorbereitung fr die TWI-Gerte
for (i = 0; i < 4; i++)							// Prfen, ob die slaves antworten, wenn ja, dann initialisieren
	{													
	chipadr = DS1631_ADR + (i * 2);			// wenn der Slave antwortet, dann Initialisierung senden
	if (TWI_Device_Seek(chipadr)) 
		{
		twi_slave_present[i] = TRUE;
		ds1631_Init(chipadr);
		j++;
		}
	else
		{
		twi_slave_present[i] = FALSE;			// nicht antwortende Slaves werden hier markiert
		}
	}

if (j == 0)											// wenn kein Slave gefunden wurde
	{
	j = 40;
	while(j)
		{
		LED_timeout = 4;
		while (LED_timeout);
		LED2_TOG;									// schnelles Blinken von LED2/3
		LED3_TOG;
		j--;
		}
	return;
	}
log_ticks = LOG_START;							// Lst nach x Sekunde den ersten Messvorgang aus
	
// --- innere Hauptschleife ----------------------------------------------------------

while(1)												// Schleife fr den Empfang von Daten
	{
	if (!(LED_timeout)) 							// nach 1/2 Sekunde ist VNC_timeout vom Timer auf 0 decrementiert
		{
		LED_timeout = 15;							// LED1 blinkt mit 1 Hz whrend des Loggens
		LED2_TOG;									
		}
			
	if (rx_flag) 									// wenn der Puffer voll ist
		{
		rx_flag = FALSE;
		bytes_received = 255;					// 255 + 1 (!) Bytes sind im Puffer
		i = (rx_buffer_ptr ^ 1);				// die Daten stehen im anderen Buffer
		DF_write_page(i);							// als Parameter wird bergeben, welcher Buffer zu schreiben ist
		LED3_TOG;
		}
														// Wenn die STart-Tasten bettigt wird, dann wird die geffnete
	if (KEY_START_PRESSED)						// Datei schieen, unvollstndige Page speichern 
		{
														// steht rx_idx auf 0, dann hat es gerade einen vollen Puffer
														// gegeben, die Daten sind gesichert, aber kein weiteres
														// Byte ist empfangen worden, daher
		if (rx_idx)									// nur wenn Bytes im Puffer sind, dann wird auch gespeichert
			{
			bytes_received = (rx_idx - 1);	// der Index ist schon fr das nchste Byte incrementiert worden !!
														// die Anzahl der empfangenen Bytes ist um 1 niedriger
			i = (rx_buffer_ptr);					// die Daten stehen im aktuellen Buffer
			DF_write_page(i);						// als Parameter wird bergeben, welcher Buffer zu schreiben ist
			}
		LED_timeout = 3;
		while (LED_timeout);
		while(KEY_START_PRESSED);				// warten bis die Taste losgelassen wird
		
		return;
		}

// Der Rest der Schleife ist fr die Abfrage der TWI-Slaves zustndig.
// Der Timer0 setzt in Abstnden das Flag LOG_flag und lst damit den Vorgang aus.
// In diesem Beispiel werden zur Temperaturmessung 4x DS1631 verwendet. 
		
// Die Temperaturmessung erfolgt in zwei Schritten:
// Zuerst wird die Messung angestossen, nach ca. 1 Sekunde kann der Messwert abgeholt werden.
// Zuerst die Messung anstossen

	if (LOG_flag)									// wird von der TIMER0_IST gesetzt
		{
		LOG_flag = FALSE;							// das Flag zurcknehmen

		LED3_ON;										// LED3 setzen, um den Logvorgang kenntlich zu machen
														// zu Irritationen kann fhren, dass das Speichern LED3 toggled !!
		for (i = 0; i < 4; i++)				
			{
			if (!(twi_slave_present[i]))		// Abbrechen, wenn der Slave nicht angemeldet ist 
				{
				continue;
				}
				
			chipadr = DS1631_ADR + (i * 2);	// fortlaufende Adresse bilden

//			if (!(TWI_Device_Seek(chipadr))) // Noch einmal die Anwesenheit prfen - gibt Probleme bei 4 Slaves, aber warum ???
//				{										// immer fr den 1. Slave wird kein Start gesendet, fr die anderen wohl
//				twi_slave_present[i] = FALSE;	// ggf. als abwesend markieren
//				}
//			else
//				{
				ds1631_Read_Start(chipadr);	// Messung starten
//				}
			}
			
		LED_timeout = 30;							// in dieser Zeit setzt das Blinken der LED2 aus	
		while (LED_timeout);						// 1 Sekunde warten

		
// nachdem eine Sekunde vorber ist, knnen die Messwerte abgeholt
// und mssen in den Tx-Buffer manipuliert werden		

		for (i = 0; i < 4; i++)					
			{
			chipadr = DS1631_ADR + (i * 2);
			if (!(twi_slave_present[i]))
				{
				save_data(i);						// Leeren Datensatz schreiben, wenn der Slave nicht vorhanden 
				save_data(chipadr);				// ohne Bedeutung, hier zum Debuggen
				save_data(255);
				save_data(255);					// an diesem Wert wird ein leerer Datensatz erkannt
				continue;							// die normalen Werte liegen zwischen 0 ... 9 (Nachkommastelle)
				}

			else										// es werden immer 4 Byte je Messung gespeichert
				{
				save_data(i);						// Nr. des Kanals speichern
				save_data(chipadr);					// ohne Bedeutung - hier zum Debuggen
				temperatur = ds1631_Read(chipadr);	// Daten auslesen
				save_data((uint8_t) (temperatur >> 8)); // Ganzzahl als INT8
				save_data((uint8_t) (temperatur)); // Nachkommastelle als UINT8
				}
			}
		LED3_OFF;									// LED3 wieder ausschalten
		}
	}
}

// ----------------------------------------------------------------------------
// legt ein Byte im Puffer ab, incrementiert den Index
// bei vollem Puffer wird umgeschaltet auf den anderen und 
// das Flag zu Absenden der Daten an den DF gesetzt
void save_data(uint8_t data)
{
rx_buffer[rx_buffer_ptr][rx_idx++] = data;

// Das Array ist 256 Byte lang, eine berlaufkontrolle ist bei 1-Byte Index also nicht erforderlich.
// Aber es muss signalisiert werden, dass die Daten nun gesendet werden mssen.

if (rx_idx == 0)
	{
	rx_flag = TRUE;								// Flag fr vollstndige Page setzen
	rx_buffer_ptr ^= 1;							// den Pointer auf den anderen EmpfangsBuffer setzen
	}

}

// ----------------------------------------------------------------------------
// spielt die letzte Aufnahme ab und sendet sie via RS232 ab.
// Nach Abschluss der bertragung wartet das Programm auf Tastenbettigung:
// Ein erneutes Drcken der PLAY-Taste gibt die nchst ltere Datei aus
// - bis keine Datei mehr gefunden wird
// Ein Bettigen der START-Taste springt ins Hauptmenue zurck.
void play(void)
{
uint16_t page, startpage, ii;
uint16_t pages_in_file;					
uint8_t file_id_low;								// low-Byte der File_id
uint8_t bytes_in_page, i;

// df_next_page zeigt auf die nchste frei Page
// den Wert kopieren, die Variable df_next_page bleibt erhalten
startpage = df_next_page;
// die letzte Aufnahme wird durch ihre ID charakterisiert.
// es muss also solange zurckgespult werden, wie die ID identisch ist.
// zustzlich ist die Anzahl der Pages ja kodiert und kann zur Berechnung des
// Anfanges herangezogen werden, wenn man die Kette von hinten her aufdrselt

LED1_OFF;											// LED1 aus
														// LED2 blinkt
LED3_ON;												// LED3 auf Dauerlicht
while(1)
	{
// das ist die letzte Page der vorhergehenden Datei
	startpage--;						 
// die Page-Nr. luft von 0, 1 bis 4094, 4095
	startpage &= PAGE_MASK;						// verhindert Over/Underflow
	
// Start dieser Datei ermitteln
// aktuelle Pagezahl - Anzahl der Pages der Datei
	DF_read_byte(startpage, 256, 8);
// Jetzt steht der Header des letzen Datensatzes in df_buffer[]
	pages_in_file  = df_buffer[P_PAGE_HIGH] << 8;
	pages_in_file += df_buffer[P_PAGE_LOW];
	
	file_id_low = df_buffer[P_ID_LOW];		// nur das low-Byte der id merken

// hier sollte der Anfang der Datei sein
	startpage -= (pages_in_file - 1) ;		// Filesize beginnt mit 1						
	startpage &= PAGE_MASK;						// verhindert Underflow

// Dorthin springen und den Header einlesen,
// wenn dort dieselbe fileid vorliegt und
// wenn filesize dort 1 ist, dann stimmt das Ziel
	DF_read_byte(startpage, 256, 8);

// die FileId (low_Byte) vergleichen, wenn sie nicht bereinstimmt,
// dann liegt ein Fehler vor.
	if (!(file_id_low == df_buffer[P_ID_LOW]))
		{
		for (i = 0; i < 30; i++)
			{
			LED_timeout = 3;
			while (LED_timeout);
			LED2_TOG;
			}
		return;
		}

// jetzt prfen, ob als filesize tatschlich 1 gefunden wird.
// wenn nein, dann liegt ein Fehler vor. 	
	if (!((df_buffer[P_PAGE_HIGH] == 0) && (df_buffer[P_PAGE_LOW] == 1)))
		{
		for (i = 0; i < 30; i++)
			{
			LED_timeout = 3;
			while (LED_timeout);
			LED2_TOG;
			}
		return;	
		}
// wenn der Eintrag fr filesize wie erwartet == 1 ist, dann die Daten ausgeben
	else
		{
// die startpage merken, fr den Fall, das weitere Dateien gelesen werden sollen
		page = startpage;
// Pageweise die Daten in den internen Buffer laden
// und via RS232 rausschreiben
// bis zum letzten Block, der wird vermutlich krzer als 256 Byte sein

		for (ii = 0; ii < pages_in_file; ii++)
			{
// die letzte Page ist wahrscheinlich nur teilweise mit Daten belegt	
			DF_read_byte(page, 256, 2);
			bytes_in_page = df_buffer[1];    // Anzahl der Bytes holen

			
			DF_read_byte(page, 256, 8);		// Druckt einzeilig den Header der Page aus
			RS232_print_header(page);			// zum Debuggen

			DF_read_byte(page, 0, 255);		// die Daten nach df_buffer[] einlesen
			RS232_print_page(bytes_in_page);	// und ausgeben
		
			page++;
			page &= PAGE_MASK;					// verhindert Overflow am Ende des verfgbaren Speichers
			
			LED2_TOG;								// LED2 toggled nach jeweils 256 Byte
			}
		}
	LED2_OFF;										// Dauerlicht nur an LED3 zeigt Ende der Ausgabe an

// warten, bis eine der beiden Tasten bettigt wird
	while (KEY_START_RELEASED && KEY_PLAY_RELEASED);

	LED_timeout = 3;
	while (LED_timeout);

// wenn die Starttaste gedrckt wird, dann zurck zur Hauptebene	
	if (KEY_START_PRESSED) 
		{	
		while(KEY_START_PRESSED);			// warten bis die Taste losgelassen wird
		return;									// dann ins Hauptmenue zurck
		}

	while(KEY_PLAY_PRESSED);				// warten bis die Play-Taste losgelassen wird
// wenn die Play-Taste gedrckt war, dann die nchste Datei ausgeben
// startpage zeigt immer noch auf den Anfang der aktuellen Datei
//	put_c('+');									// zum Debuggen
//	put_c(13);
	}
}

// ----------------------------------------------------------------------------
// TIMER0_OVF:  3.686400 MHZ / (1024 * 120) -> 33ms  Zhlt VNC_timeout herunter auf 0
// als Grundlage fr die Zeitzhlung muss ein Takt gewhlt werden, aus dem sich
// der Sekundentakt ableiten lsst: Die ISR wird 30x /Sekunde aufgerufen
ISR(TIMER0_COMPA_vect)
{
static uint8_t ticks;							// der Startwert ist gleichgltig
//static uint8_t log_ticks = 1;	 			// ist global geworden

if(LED_timeout) LED_timeout--;				// timeout runterzhlen

ticks--;
if (!(ticks))										// wenn 1 Sekunde um ist (tick = 30)
	{												
	ticks = 30;
	time++;											// die Zeit im Sekundentakt incrementieren
	
														// --- das Messintervall 
	log_ticks--;									// jede Sekunde wird der Wert decrementiert - bis auf 0
	if (!(log_ticks))
		{
		log_ticks = MESSINTERVAL;				// das Messintervall in Sekunden
		LOG_flag  = TRUE;							
		}
	}
}


// --- [ eof ] ----------------------------------------------------------------

